﻿using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using Dlubal.RFEM5;
using Grasshopper.Kernel.Types;

namespace rFER_ExcelExp
{
    class RFEM_Connect
    {

        public void CreateNodes(List<GH_Point> kList) // Funktioniert nicht ;-(  
        {
            IModel model = Marshal.GetActiveObject("RFEM5.Model") as IModel;
            model.GetApplication().LockLicense();
            IModelData data = model.GetModelData();
            
            List<Node> NodeList = new List<Node>();
            foreach (GH_Point p in kList)
            {
                Node n = new Node();
                n.X = p.Value.X;
                n.Y = p.Value.Y;
                n.Z = p.Value.Z;
                NodeList.Add(n);
            }
            Node[] NodeArray = NodeList.ToArray();

            data.PrepareModification();
            data.SetNodes(NodeArray);
            data.FinishModification();
            model.GetApplication().UnlockLicense();
        }

        public void CreateNode(GH_Point p)
        {
            IModel model = Marshal.GetActiveObject("RFEM5.Model") as IModel;
            model.GetApplication().LockLicense();
            IModelData data = model.GetModelData();

            Node node = new Node();
            node.X = p.Value.X;
            node.Y = p.Value.Y;
            node.Z = p.Value.Z;

            data.PrepareModification();
            try { data.SetNode(node); }
            catch (Exception) { }
            data.FinishModification();
            model.GetApplication().UnlockLicense();
        }

        public void CreateLine(string nList, string type, string comment)
        {
            IModel model = Marshal.GetActiveObject("RFEM5.Model") as IModel;
            model.GetApplication().LockLicense();
            IModelData data = model.GetModelData();

            Line line = new Line();
            line.NodeList = nList;
            line.Comment = comment;
            line.Type = LineType.PolylineType;
            if (type == "NURBS")
            {
                line.Type = LineType.BSplineType;
            }

            data.PrepareModification();
            //data.SetLine(line);
            try { data.SetLine(line); }
            catch (Exception) { }
            data.FinishModification();
            model.GetApplication().UnlockLicense();
        }

        public void CreateSurface(string lineList, string AlignLine, int aNr, int fNr)
        {
            IModel model = Marshal.GetActiveObject("RFEM5.Model") as IModel;
            model.GetApplication().LockLicense();
            IModelData data = model.GetModelData();

            Surface surface = new Surface();
            surface.BoundaryLineList = lineList;
            surface.MaterialNo = 1;
            SurfaceThickness sTh = new SurfaceThickness();
            sTh.Constant = 1.0;
            sTh.Type = SurfaceThicknessType.ConstantThicknessType;
            surface.Thickness = sTh;

            SurfaceAxes sAxes = new SurfaceAxes();
            sAxes.No = aNr;
            sAxes.Direction = SurfaceAxesDirection.SurfaceAxisXParallelToLine;
            sAxes.LineList = AlignLine;

            data.PrepareModification();
            try { data.SetSurface(surface); }
            catch (Exception) { }
            data.FinishModification();
            model.GetApplication().UnlockLicense();

            data.PrepareModification();
            try { data.GetSurface(fNr, ItemAt.AtNo).SetInputAxes(sAxes); }
            catch (Exception) { }
            data.FinishModification();
            model.GetApplication().UnlockLicense();
        }

        public void CreateQuad(string lineList)
        {
            IModel model = Marshal.GetActiveObject("RFEM5.Model") as IModel;
            model.GetApplication().LockLicense();
            IModelData data = model.GetModelData();

            Surface surface = new Surface();
            surface.GeometryType = SurfaceGeometryType.QuadrangleSurfaceType;
            surface.BoundaryLineList = lineList;
            surface.MaterialNo = 1;
            SurfaceThickness sTh = new SurfaceThickness();
            sTh.Constant = 1.0;
            sTh.Type = SurfaceThicknessType.ConstantThicknessType;
            surface.Thickness = sTh;

            data.PrepareModification();
            //data.SetSurface(surface);
            try { data.SetSurface(surface); }
            catch (Exception) { }
            data.FinishModification();
            model.GetApplication().UnlockLicense();
        }

        public void CreateSurfaceBspline(string lineList)
        {
            IModel model = Marshal.GetActiveObject("RFEM5.Model") as IModel;
            model.GetApplication().LockLicense();
            IModelData data = model.GetModelData();

            Surface surface = new Surface();
            surface.GeometryType = SurfaceGeometryType.BSplineSurfaceType;
            surface.BoundaryLineList = lineList;
            surface.MaterialNo = 1;
            SurfaceThickness sTh = new SurfaceThickness();
            sTh.Constant = 1.0;
            sTh.Type = SurfaceThicknessType.ConstantThicknessType;
            surface.Thickness = sTh;

            data.PrepareModification();
            //data.SetSurface(surface);
            try { data.SetSurface(surface); }
            catch (Exception) { }
            data.FinishModification();
            model.GetApplication().UnlockLicense();
        }

        public void CreateSolid(string sList, int solidNr, int surfOffset)
        {
            IModel model = Marshal.GetActiveObject("RFEM5.Model") as IModel;
            model.GetApplication().LockLicense();
            IModelData data = model.GetModelData();
            Solid solid = new Solid();

            solid.SurfaceList = sList;
            solid.Type = SolidType.MaterialSolidType;
            solid.MaterialNo = 1;

            SolidAxes sAxes = new SolidAxes();
            sAxes.Direction = SolidAxesDirection.SolidAxesXYParallelToSurface;
            sAxes.SurfaceNo = surfOffset;

            data.PrepareModification();
            try
            {
                data.SetSolid(solid);
                data.GetSolid(solidNr, ItemAt.AtNo).SetAxes(sAxes);
            }
            catch (Exception) { }
            data.FinishModification();
            model.GetApplication().UnlockLicense();
        }

    }
}
